﻿B4R=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=3
@EndOfDesignText@
Private Sub Process_Globals
    'These global variables will be declared once when the application starts.
    'Public variables can be accessed from all modules.
 
    Private pclk, pdat, prst As Pin          'clock, I/O, CE
    Private data As Byte = 0
	Private thisDAT As Byte = 0
	
	Private originalCommand As Byte = 0
    Private morning As Boolean = False
	Private mode24 As Boolean = True
 
End Sub

'Initialize the timer with the required pins
'clk = pin number of arduino that the DS1302 CLK pin in connected to (serial Clock)
'dat = pin number of arduino that the DS1302 DAT pin in connected to (I/O or Data)
'rst = pin number of arduino that the DS1302 RST pin in connected to (Reset or Chip Enable)
'Note - The pins on the DS1302 might be labelled as CE or RST, I/O or DAT(data line), and SCLK or CLK (serial clock).
'CE = RST
'I/O = DAT
'SCLK = CLK
Public Sub Initialize (clk As Byte, dat As Byte, rst As Byte)
 
    pclk.Initialize(clk, pclk.MODE_OUTPUT)
    pdat.Initialize(dat, pdat.MODE_OUTPUT)
    prst.Initialize(rst, prst.MODE_OUTPUT)
	thisDAT = dat
 
    pclk.DigitalWrite(False)
    pdat.DigitalWrite(False)
    prst.digitalwrite(False)
 
End Sub

'Set the Minute of the DS1302
'The method will automatically clock in 0x82 (BCD) as a command to set the Minutes
Public Sub setMinutes (minutes As Byte)

    Dim command As Byte = 0x82                   'command to set the minutes = 0x82h (BCD)
    sendCommand(command)
    setValue(minutes)
 
End Sub

'Set the Hour of the DS1302
'The method will automatically clock in 0x84 (BCD) as a command to set the Hours
Public Sub setHours24(hours As Byte)
 
    Dim command As Byte = 0x84                   'command to set the Hours = 0x84h (BCD)
    sendCommand(command)
    setValue(hours)
 
End Sub


'Set the Hour of the DS1302 (12hr format)
'The method will automatically clock in 0x84 (BCD) as a command to set the Hours
'Set AM = True if hours is AM
'Set AM = False if hours is PM 
Public Sub setHours12(hours As Byte, AM As Boolean)
 
	Dim command As Byte = 0x84                   'command to set the Hours = 0x84h (BCD)
	sendCommand(command)                         'send command to the DS1302 to set the hours
 
    Dim hour10 As Byte = hours/10                'get the 10's of the hours
	Dim hour1 As Byte = hours Mod 10             'get the 1's of the hours
    morning = AM                                 'set the global boolean variable - we will need it later
    If morning = False Then                      'we are setting time as PM
		Dim mask As Byte = 10                    'mask to set the 12HR/24HR bit HIGH and also set the AM/PM bit HIGH
		hour10 = Bit.Or(hour10, mask)
		hours = hour10 * 10 + hour1
	Else	                                     'we are setting time as AM
		Dim mask As Byte = 8                     'we need to set the 12HR/24HR bit to HIGH so that time will be in 12HR format. It also sets the AM/PM bit LOW for AM
		hour10 = Bit.Or(hour10, mask)            'set the bit
		hours = hour10 * 10 + hour1              'calculate the decimal value for "hours" that we need to pass to Sub setValue
	End If   
	
	setValue(hours)                               'call Sub setValue and pass the decimal value. 
 
End Sub

'Set the Month of the DS1302
'The method will automatically clock in 0x88 (BCD) as a command to set the Month
'January = 1, February = 2, ........, December = 12
Public Sub setMonth(month As Byte)
 
    Dim command As Byte = 0x88                   'command to set the Month = 0x88h (BCD)
    sendCommand(command)
    setValue(month)
 
End Sub

'Set the DayOfWeek of the DS1302
'The method will automatically clock in 0x8A (BCD) as a command to set the Day Of Week
'Sunday = 1, Monday = 2, ........, Saturday = 7
Public Sub setDayOfWeek(dow As Byte)
 
    Dim command As Byte = 0x8A                   'command to set the Month = 0x8Ah (BCD)
    sendCommand(command)
    setValue(dow)
 
End Sub

'Set the DayOfMonth of the DS1302
'The method will automatically clock in 0x86 (BCD) as a command to set the Day Of Month
'Day Of Month = 1...31
Public Sub setDayOfMonth(dom As Byte)
 
    Dim command As Byte = 0x86                   'command to set the Day Of Month = 0x86h (BCD)
    sendCommand(command)
    setValue(dom)
 
End Sub

'Set the Year of the DS1302
'The method will automatically clock in 0x8C (BCD) as a command to set the Year
'Year = 2000...2099
'The DS1302 only allows for years from 2000 to 2099 so no need to send the century - only send the Year of the century.
'eg: setTimeAndDate1302.setYear(20)
Public Sub setYear(year As Byte)
 
    Dim command As Byte = 0x8C                   'command to set the Year = 0x8Ch (BCD)
    sendCommand(command)
    setValue(year)
 
End Sub

'Set the Seconds of the DS1302
'The method will automatically clock in 0x80 (BCD) as a command to set the Seconds
'Seconds = 0...59
Private Sub setSeconds(seconds As Byte)
 
    Dim command As Byte = 0x80                   'command to set the Year = 0x80h (BCD)
    sendCommand(command)
    setValue(seconds)
 
End Sub

'set the RTC date and time in one GO - it will be set to 24HR format
'Year = 0 - 99 (which represents 2000 to 2099 - the DS1302 only allows for these years)
'month = 1 - 12 (1 = January,......,12 = December)
'dayOfMonth = 0 - 31 (take care of what you do if the month is for eg February and it is/is not a leap year)
'dayOfWeek = 1 - 7 (Sunday = 1, Monday = 2, ......, Saturday = 7)
'hour = 0 - 23
'minute = 0 - 59
Public Sub setRTCTime(year As Byte, month As Byte, dayOfMonth As Byte, dayOfWeek As Byte, hour As Byte, minute As Byte, second As Byte)
 
    setSeconds(second)
    setMinutes(minute)
    setHours24(hour)
    setDayOfMonth(dayOfMonth)
    setMonth(month)
    setDayOfWeek(dayOfWeek)
    setYear(year)
 
End Sub

'Common Private Sub to send the necessary command to the DS1302 before setting the applicable RTC register
Private Sub sendCommand (command As Byte)

	originalCommand = command

    prst.Digitalwrite(True)                      'pull CE line high                  '
    DelayMicroseconds(2)
 
    Dim mask As Byte = 1                         'set the mask to AND the command with
    For i = 0 To 7                               'cycle through the 8 bits of the command
        If Bit.And(command, mask) = 1 Then
            pdat.DigitalWrite(True)              'take the DAT line high
        Else
            pdat.DigitalWrite(False)             'take the DAT line low
        End If
        DelayMicroseconds(1)
        pclk.DigitalWrite(True)                  'take the CLK line high
        DelayMicroseconds(1)
        pclk.DigitalWrite(False)                 'take the CLK line low
        DelayMicroseconds(1)
        command = Bit.ShiftRight(command,1)      'shift the command bits 1 bit to the right for the next AND instruction
    Next
 
End Sub

'Common Private Sub to send the necessary data to the DS1302 to set the applicable RTC register
private Sub setValue(value As Byte)
 
    If originalCommand < 0xC0 Then               'i.e not writing to one of the RAM locations but to a time register
	    Dim highnibble As Byte = value/10
	    highnibble = Bit.ShiftLeft(highnibble,4)
	    Dim lownibble As Byte = value Mod 10
	    value = Bit.Or(highnibble,lownibble)
    End If
    For i = 0 To 7
        If Bit.And(value, 1) = 1 Then
            pdat.DigitalWrite(True)              'take the DAT line high
        Else
            pdat.DigitalWrite(False)             'take the DAT line low
        End If
        DelayMicroseconds(1)
        pclk.DigitalWrite(True)                  'take the CLK line high
        DelayMicroseconds(1)
        pclk.DigitalWrite(False)                 'take the CLK line low
        DelayMicroseconds(1)
        value = Bit.ShiftRight(value,1)          'shift the command bits 1 bit to the right for the next AND instruction
    Next
    prst.DigitalWrite(False)                     'pull CE line low
 
End Sub

'Common Private Sub to get Time/Date data from the DS1302
Private Sub getdata (command As Byte) As Byte

	originalCommand = command
    
    prst.Digitalwrite(True)                      'pull CE line high                  '
    DelayMicroseconds(2)
 
    Dim mask As Byte = 1                         'set the mask to AND the command with
    For i = 0 To 7                               'cycle through the 8 bits of the command
        If Bit.And(command, mask) = 1 Then
            pdat.DigitalWrite(True)              'take the DAT line high
        Else
            pdat.DigitalWrite(False)             'take the DAT line low
        End If
        DelayMicroseconds(1)
        pclk.DigitalWrite(True)                  'take the CLK line high
        DelayMicroseconds(1)
        pclk.DigitalWrite(False)                 'take the CLK line low
        DelayMicroseconds(1)
        command = Bit.ShiftRight(command,1)      'shift the command bits 1 bit to the right for the next AND instruction
    Next
 
    data = 0
 
	pdat.Initialize(thisDAT, pdat.MODE_INPUT)    'set the I/O line to mode input to read the response from the RTC (DS1302) module

    mask = 1

    For i = 0 To 7                                                 'cycle through the response from the RTC module
        If pdat.DigitalRead = True Then data = Bit.Or(data, mask)  'check if the DAT pin (response) is HIGH/LOW and set byte "data" accordingly
        DelayMicroseconds(1)
        pclk.DigitalWrite(True)                                    'take the CLK pin high
        DelayMicroseconds(1)
        pclk.DigitalWrite(False)                                   'take the CLK pin LOW
        DelayMicroseconds(1)
        mask = mask * 2                                            'double "mask" to check the next bit read from the RTC module
    Next
 
    prst.DigitalWrite(False)                     'pull CE line low

	If originalCommand = 0x85 Then               'if we have requested to read the byte that contains HOUR information from the DS1302 -> we have sent 0x85 as a command
		check24HR12HRAmPm
	End If
	
	Dim val10, val1, val As Byte = 0
	If originalCommand < 0xc0 Then               'i.e not reading a value from one of the RAM locations but from a time register
	    val10 = 0
	    val1 = 0
	    val1 = Bit.And(data, 0x0f)               'get the ONE's value of the lower 4 bits in data
	    val10 = Bit.And(data, 0xf0)              'get the TEN's value of the lower 4 bits in data
	    val10 = Bit.ShiftRight(val10,4)          'shift the TENS nibble 4 bits to the right
	    val = (val10 * 10) + val1                'the final value to be returned
	Else
		val = data
	End If
	pdat.Initialize(thisDAT, pdat.MODE_OUTPUT)   'take the I/O line back to output mode so that we can write a new/next command to it
    Return val                                   'return the calculated value

End Sub


Private Sub check24HR12HRAmPm
	
	Dim mask As Byte
'	If originalCommand = 0x85 Then               'we have requested to read the byte that contains HOUR information from the DS1302 -> we have sent 0x85 as a command
		mask = 128                               'mask to check if hours is set to 12HR or 24HR mode
		If Bit.And(data, mask) = 0 Then          'if the bit = 0 then we are in 24HR mode
			mode24 = True
		End If
		If Bit.And(data, mask) = mask Then
			mode24 = False                       'if the bit != 0 then we are in 12HR mode
		End If
		mask = 32                                'mask to check if hours is AM or PM at present
		If Bit.And(data, mask) = 0 And mode24 = False Then
			morning = True                       'if the bit = 0 and we are in 12HR mode then it is AM i.e morning = True
		End If
		If Bit.And(data, mask) = mask And mode24 = False Then
			morning = False                      'if the bit != 0 and we are in 12HR mode then it is PM i.e morning = False
		End If
		
		If mode24 = True Then
			mask = 63                            'mask to filter out AM/PM and 12HR/24HR bits so that we can get the correct hour value
			data = Bit.And(data, mask)
		Else
			mask = 31                            'mask to filter out AM/PM and 12HR/24HR bits so that we can get the correct hour value
			data = Bit.And(data, mask)
		End If		


'	End If
	
End Sub

'get the seconds - the metod will automatically clock in 81h from LSB to MSB (1000 0001) as a command to get the seconds (0...59)
Public Sub getSeconds() As Byte
    Return getdata(0x81)      'get the seconds - clock in 81h from LSB to MSB (1000 0001)
End Sub

'get the minutes - the metod will automatically clock in 83h from LSB to MSB (1000 0011) as a command to get the minutes (0...59)
Public Sub getMinutes() As Byte
    Return getdata(0x83)      'get the minutes - clock in 83h from LSB to MSB (1000 0011)
End Sub

'get the hours - the metod will automatically clock in 85h from LSB to MSB (1000 0101) as a command to get the hours (0...23)
Public Sub getHours() As Byte
    Return getdata(0x85)      'get the hours - clock in 85h from LSB to MSB (1000 0101)
End Sub

'get the year - the metod will automatically clock in 8Dh from LSB to MSB (1000 1101) as a command to get the year (0...99)
Public Sub getYear() As Byte
    Return getdata(0x8D)      'get the year - clock in 8Dh from LSB to MSB (1000 1101)
End Sub

'get the month - the metod will automatically clock in 89h from LSB to MSB (1000 1001) as a command to get the month (1...12)
Public Sub getMonth() As Byte
    Return getdata(0x89)      'get the month - clock in 89h from LSB to MSB (1000 1001)
End Sub

'get the day of the month - the metod will automatically clock in 87h from LSB to MSB (1000 0111) as a command to get the day of month (0...31)
Public Sub getDayOfMonth() As Byte
    Return getdata(0x87)      'get the day of the month - clock in 87h from LSB to MSB (1000 0111)
End Sub

'get the day number of the week - the metod will automatically clock in 8Bh from LSB to MSB (1000 1011) as a command to get the day of week (1...7)
Public Sub getDayOfWeek() As Byte
    Return getdata(0x8B)      'get the day number of the week - clock in 8Bh from LSB to MSB (1000 1011)
End Sub

'Get the DS1302 12HR/24HR clock setting
'return True if 24HR mode
'return False if 12HR mode
Public Sub getHourMode() As Boolean
	Return mode24	
End Sub

'Get the DS1302 AM/PM clock setting
'return True if it is AM
'return False if it is PM
'it only returns a useful value (True / False) if the 12HR/24HR mode is set to 12HR
Public Sub getAmPmMode() As Boolean
	Return morning	
End Sub

'The static RAM is 31 x 8 bytes addressed consecutively in the RAM address space.
'value is a byte (0 to 255) to be stored is the "address" RAM location
'write addresses/commands are from 0xC0 to 0xFC in steps of 2 i.e 0xC2, 0xC2, 0xC4, ......, 0xFC (for write)
'store a byte value in the specified RAM address
Public Sub writeValueToRamAddress (address As Byte, value As Byte)
	
	sendCommand(address)
	setValue(value)
	
End Sub

'The static RAM is 31 x 8 bytes addressed consecutively in the RAM address space.
'read addresses/commands are from 0xC1 to 0xFD in steps of 2 i.e 0xC1, 0xC3, 0xC5, ......, 0xFD (for read)
'returns the value store in the specified address 
Public Sub readValueFromRamAddress (address As Byte) As Byte
	
	Return getdata(address)
	
End Sub